IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceGetInternalIdentifiers'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceGetInternalIdentifiers
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceGetInternalIdentifiers]
(
  @createIfNotFound BIT
, @entity           NVARCHAR(6)
, @mrn              NVARCHAR(12)
, @admNo            NVARCHAR(12)
, @vstNo            NVARCHAR(12)
, @enterpriseID     NVARCHAR(12) = N''
, @oldAdmNo         NVARCHAR(12) = N''
)
AS
BEGIN

  DECLARE @enterpriseIDExists  BIT
  DECLARE @mrnExists           BIT
  DECLARE @admExists           BIT
  DECLARE @enterpriseOID       INT
  DECLARE @mrnOID              INT
  DECLARE @enterpriseEntityMRN NVARCHAR(12)
  DECLARE @mrnEnterpriseID     NVARCHAR(12)
  DECLARE @admitOID            INT
  DECLARE @admitPatientOID     INT
  DECLARE @patientOID          INT
  DECLARE @visitOID            INT

  SELECT
    @enterpriseIDExists    = 0
    , @mrnExists           = 0
    , @admExists           = 0
    , @enterpriseOID       = NULL
    , @mrnOID              = NULL
    , @enterpriseEntityMRN = NULL
    , @mrnEnterpriseID     = NULL
    , @admitOID            = NULL
    , @admitPatientOID     = NULL
    , @patientOID          = NULL
    , @visitOID            = NULL

  -- Read the PatientOID associated to the enterpriseID passed
  --  along with the MRN associated to that PatientOID in the
  --  entity this request is for.
  SELECT
    @enterpriseOID       = ENTERPRISE.IdentifierOID
   ,@enterpriseEntityMRN = ENTITY.ID
  FROM
    PatientIdentifiers ENTERPRISE
      LEFT OUTER JOIN PatientIdentifiers ENTITY
        ON  ENTITY.IdentifierOID = ENTERPRISE.IdentifierOID
        AND ENTITY.IdentifierType = 'MRN'
        AND ENTITY.entity = @entity
  WHERE
    ENTERPRISE.ID = @enterpriseID
    AND ENTERPRISE.IdentifierType = 'ENT'

  IF (@enterpriseOID IS NOT NULL)
  BEGIN
    SELECT @enterpriseIDExists = 1
  END

  -- Read the PatientOID associated to the MRN/Entity passed
  --  along with the enterpriseID associated to that PatientOID
  SELECT
    @mrnOID          = ENTITY.IdentifierOID
   ,@mrnEnterpriseID = ENTERPRISE.ID
  FROM
    PatientIdentifiers ENTITY
      LEFT OUTER JOIN PatientIdentifiers ENTERPRISE
        ON ENTERPRISE.IdentifierOID = ENTITY.IdentifierOID
        AND ENTERPRISE.IdentifierType = 'ENT'
  WHERE
    ENTITY.ID = @mrn
    AND ENTITY.IdentifierType = 'MRN'
    AND ENTITY.entity = @entity

  IF (@mrnOID IS NOT NULL)
  BEGIN
    SELECT @mrnExists = 1
  END

  -- CHECKS - Patient Level
  --   1)  EnterpriseID exists
  --     a)  MRN exists
  --       .  EnterpriseMRN = MRN
  --       .  MRNEnterpriseID = EnterpriseID
  --       .  EnterpriseOID = MRNOID
  --     b)  MRN does NOT exist
  --       .  EnterpriseEntityMRN IS NULL
  --   2)  Enterprise does NOT exist
  --     a)  MRN exists
  --       .  MRNEnterpriseID IS NULL
  IF (@enterpriseIDExists = 1)
  BEGIN
    IF (@mrnExists = 1)
    BEGIN
      -- MRN does exist
      IF (@enterpriseEntityMRN != @mrn)
      BEGIN
        -- MRN tied to the PatientOID the EnterpriseID
        --   in the passed Entity is NOT equal to the MRN
        --   passed.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, MRN (%s) assigned to EnterpriseID (%s) in Entity (%s) <> MRN (%s) passed.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@enterpriseEntityMRN
                  ,@enterpriseID
                  ,@entity
                  ,@mrn); 
      END

      IF (@mrnEnterpriseID != @enterpriseID)
      BEGIN
        -- EnterpriseID tied to the PatientOID of the
        --   passed MRN/Entity is NOT equal to the EnterpriseID
        --   passed.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, EnterpriseID (%s) assigned to MRN (%s) in Entity (%s) <> EnterpriseID (%s) passed.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@mrnEnterpriseID
                  ,@enterpriseEntityMRN
                  ,@entity
                  ,@enterpriseID); 
      END

      IF (@enterpriseOID != @mrnOID)
      BEGIN
        -- EnterpriseID is tied to a different PatientOID
        --   than the MRN is in the passed entity.  This
        --   request would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, EnterpriseID (%s) assigned to PatientOID <> PatientOID assigned to MRN (%s) in Entity (%s).'
                  ,0 -- Severity
                  ,1 -- State
                  ,@enterpriseID
                  ,@mrn
                  ,@entity);
      END

      -- To get here both the enterpriseID and MRN were found and are
      --  tied to the same PatientOID.  Set the variable that we will
      --  be returning (can be assigned to either one.
      SELECT @patientOID = @mrnOID
    END
    ELSE
    BEGIN
      -- MRN does NOT exist
      IF (@enterpriseEntityMRN IS NOT NULL)
      BEGIN
        -- MRN tied to the PatientOID the EnterpriseID
        --   in the passed Entity exists.  This can only
        --   happen if it's different than the passed
        --   MRN.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, passed MRN (%s) in Entity (%s) NOT found, but EnterpriseID (%s) has MRN (%s) assigned in that entity.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@mrn
                  ,@entity
                  ,@enterpriseID
                  ,@enterpriseEntityMRN);
      END

      -- EnterpriseID was found, we'll want to use the PatientOID
      --  that ID is assigned to to add MRN/Entity on.  Set the
      --  variable that we will be returning.
      SELECT @patientOID = @enterpriseOID
    END
  END
  ELSE
  BEGIN
    IF (@mrnExists = 1)
    BEGIN
      -- MRN does exist
      IF (@mrnEnterpriseID IS NOT NULL)
      BEGIN
        -- EnterpriseID tied to the PatientOID of the MRN
        --   /Entity passed Entity exists.  This can only
        --   happen if it's different than the passed
        --   EnterpriseID.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, passed EnterpriseID (%s) NOT found, but MRN (%s) in Entity (%s) has EnterpriseID (%s) assigned.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@enterpriseID
                  ,@mrn
                  ,@entity
                  ,@mrnEnterpriseID);
      END

      -- MRN/Entity was found, we'll want to use the PatientOID
      --  that MRN is assigned to to add the EnterpriseID on.  Set the
      --  variable that we will be returning.
      SELECT @patientOID = @mrnOID
    END
    ELSE
    BEGIN
      -- MRN/EnterpriseID do NOT exist
      IF (@createIfNotFound = 1)
      BEGIN
        --  Generate a NEWID to return to the caller to be used.
        EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'Patient', @nextOID = @patientOID OUTPUT
      END
      ELSE
      BEGIN
        SELECT @patientOID = 0
      END
    END
  END

  -- Search for the patient account # passed
  SELECT
    @admitOID        = ADM_ID.IdentifierOID
   ,@admitPatientOID = VISIT.PatientOID
  FROM
    PatientIdentifiers ADM_ID
      INNER JOIN PatientIdentifiers VST_ID
        ON VST_ID.IdentifierOID = ADM_ID.IdentifierOID
          AND VST_ID.Entity = ADM_ID.Entity
      LEFT OUTER JOIN PatientVisits VISIT
        ON VISIT.VisitOID = ADM_ID.IdentifierOID
          AND VISIT.Entity = ADM_ID.Entity
  WHERE
    ADM_ID.ID = @admNo
    AND ADM_ID.IdentifierType = N'ADM'
    AND VST_ID.ID = @vstNo
    AND VST_ID.IdentifierType = N'VST'
    AND ADM_ID.Entity = @entity

  IF (@admitOID IS NOT NULL)
  BEGIN
    SELECT @admExists = 1
  END
  ELSE
  BEGIN
    -- Patient Status Transfer - The patient account # passed does NOT exist
    --   yet - we're going to update the old account # to that.  Search for
    --   the visit by the old account # we'll be updating.
    SELECT
      @admitOID        = ids.IdentifierOID
     ,@admitPatientOID = VISIT.PatientOID
    FROM
      PatientIdentifiers ids
        LEFT OUTER JOIN PatientVisits VISIT
          ON VISIT.VisitOID = ids.IdentifierOID
            AND VISIT.Entity = ids.Entity
    WHERE
      ids.ID = @oldAdmNo
      AND ids.IdentifierType = 'ADM'
      AND ids.Entity = @entity

    IF (@admitOID IS NOT NULL)
    BEGIN
      SELECT @admExists = 1
    END
  END

  -- CHECKS - Visit Level
  --   1)  AdmNo exists
  --     a)  MRN exists
  --       .  AdmNo PatientOIOD = MRNOID
  --     b)  MRN does NOT exist
  --       .  AdmNo PatientOIOD IS NULL
  --   2)  AdmNo does NOT exist
  --     a)  No check
  IF (@admExists = 1)
  BEGIN
    IF (@mrnExists = 1)
    BEGIN
      -- MRN does exist
      IF (@admitPatientOID != @mrnOID)
      BEGIN
        -- Admission tied to the PatientOID of a differnet MRN
        --   /Entity than passed.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, passed Admission # (%s) tied to different PatientOID than MRN (%s) in Entity (%s) passed.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@admNo
                  ,@mrn
                  ,@entity);
      END
    END
    ELSE
    BEGIN
      -- MRN does NOT exist
      IF (@admitPatientOID IS NOT NULL)
      BEGIN
        -- Admission tied to the PatientOID of a differnet MRN
        --   /Entity than passed.  This would cross patient information.
        -- RAISEERROR to alert the caller - FATAL
        RAISERROR (N'Crossed patient identifiers, passed Admission # (%s) tied to a patient, but MRN (%s) in Entity (%s) NOT found.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@admNo
                  ,@mrn
                  ,@entity);
      END
    END

    -- AdmNo was found and passed all the checks maded
    --   against the mrn.  Return the VisitOID found to
    --   the caller
    SELECT @visitOID = @admitOID
  END
  ELSE
  BEGIN
    -- AdmNo does NOT exist
    IF (@createIfNotFound = 1)
    BEGIN
      --  Generate a NEWID to return to the caller to be used.
      EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'Visit', @nextOID = @visitOID OUTPUT
    END
    ELSE
    BEGIN
      SELECT @visitOID = 0
    END
  END

  SELECT
   @mrnExists           AS 'MrnExists'
   ,@enterpriseIDExists AS 'EnterpriseIDExists'
   ,@admExists          AS 'AdmNoExists'
   ,@entity             AS 'Entity'
   ,@patientOID         AS 'PatientOID'
   ,@visitOID           AS 'VisitOID'

END
GO
